@@ -132,11 +132,11 @@ func (c *Composer) SelectedDirectory() string {
 	return c.seldir
 }
 
+func (c *Composer) Parent() *models.OriginalMail {
+	return c.parent
+}
+
 func (c *Composer) SwitchAccount(newAcct *AccountView) error {
-	if c.acct == newAcct {
-		log.Tracef("same accounts: no switch")
-		return nil
-	}
 	// sync the header with the editors
 	for _, editor := range c.editors {
 		editor.storeValue()
@@ -542,7 +542,14 @@ func (c *Composer) setContents(reader io.Reader) error {
 				}
 			}
 			key := textproto.CanonicalMIMEHeaderKey(h)
-			_, err = fmt.Fprintf(c.email, "%s: %s"+lineEnding, key, value)
+
+			var sep string
+			if value == "" {
+				sep = ":"
+			} else {
+				sep = ": "
+			}
+			_, err = fmt.Fprintf(c.email, "%s%s%s%s", key, sep, value, lineEnding)
 			if err != nil {
 				return err
 			}
@@ -781,9 +788,13 @@ func (c *Composer) MouseEvent(localX int, localY int, event vaxis.Event) {
 	for i, e := range c.focusable {
 		he, ok := e.(*headerEditor)
 		if ok && he.focused {
-			c.focusActiveWidget(false)
-			c.focused = i
-			c.focusActiveWidget(true)
+			if c.editor == nil {
+				he.focused = false
+			} else {
+				c.focusActiveWidget(false)
+				c.focused = i
+				c.focusActiveWidget(true)
+			}
 			return
 		}
 	}
@@ -791,13 +802,15 @@ func (c *Composer) MouseEvent(localX int, localY int, event vaxis.Event) {
 
 func (c *Composer) Focus(focus bool) {
 	c.Lock()
-	c.focusActiveWidget(focus)
+	if c.editor != nil {
+		c.focusActiveWidget(focus)
+	}
 	c.Unlock()
 }
 
 func (c *Composer) Show(visible bool) {
 	c.Lock()
-	if w := c.focusedWidget(); w != nil {
+	if w := c.focusedWidget(); w != nil && c.editor != nil {
 		if vis, ok := w.(ui.Visible); ok {
 			vis.Show(visible)
 		}
@@ -1184,7 +1197,15 @@ func (c *Composer) reopenEmailFile() error {
 
 func (c *Composer) termClosed(err error) {
 	c.Lock()
-	defer c.Unlock()
+	// RemoveTab() on error must be called *AFTER* c.Unlock() but the defer
+	// statement does the exact opposite (last defer statement is executed
+	// first). Use an explicit list that begins with unlocking first.
+	deferred := []func(){c.Unlock}
+	defer func() {
+		for _, d := range deferred {
+			d()
+		}
+	}()
 	if c.editor == nil {
 		return
 	}
@@ -1192,7 +1213,7 @@ func (c *Composer) termClosed(err error) {
 		PushError("Failed to reopen email file: " + e.Error())
 	}
 	editor := c.editor
-	defer editor.Destroy()
+	deferred = append(deferred, editor.Destroy)
 	c.editor = nil
 	c.focusable = c.focusable[:len(c.focusable)-1]
 	if c.focused >= len(c.focusable) {
@@ -1200,8 +1221,10 @@ func (c *Composer) termClosed(err error) {
 	}
 
 	if editor.cmd.ProcessState.ExitCode() > 0 {
-		RemoveTab(c, true)
-		PushError("Editor exited with error. Compose aborted!")
+		deferred = append(deferred, func() {
+			RemoveTab(c, true)
+			PushError("Editor exited with error. Compose aborted!")
+		})
 		return
 	}
 
@@ -1212,8 +1235,10 @@ func (c *Composer) termClosed(err error) {
 			PushError(err.Error())
 			err := c.showTerminal()
 			if err != nil {
-				RemoveTab(c, true)
-				PushError(err.Error())
+				deferred = append(deferred, func() {
+					RemoveTab(c, true)
+					PushError(err.Error())
+				})
 			}
 			return
 		}
@@ -1276,17 +1301,17 @@ func (c *Composer) showTerminal() error {
 	c.focusable = append(c.focusable, c.editor)
 	c.review = nil
 	c.updateGrid()
-	if c.editHeaders {
+	if c.editHeaders || config.Compose.FocusBody {
 		c.focusTerminalPriv()
 	}
 	return nil
 }
 
-func (c *Composer) PrevField() {
+func (c *Composer) PrevField() bool {
 	c.Lock()
 	defer c.Unlock()
-	if c.editHeaders && c.editor != nil {
-		return
+	if c.editHeaders || c.editor == nil {
+		return false
 	}
 	c.focusActiveWidget(false)
 	c.focused--
@@ -1294,39 +1319,42 @@ func (c *Composer) PrevField() {
 		c.focused = len(c.focusable) - 1
 	}
 	c.focusActiveWidget(true)
+	return true
 }
 
-func (c *Composer) NextField() {
+func (c *Composer) NextField() bool {
 	c.Lock()
 	defer c.Unlock()
-	if c.editHeaders && c.editor != nil {
-		return
+	if c.editHeaders || c.editor == nil {
+		return false
 	}
 	c.focusActiveWidget(false)
 	c.focused = (c.focused + 1) % len(c.focusable)
 	c.focusActiveWidget(true)
+	return true
 }
 
-func (c *Composer) FocusEditor(editor string) {
+func (c *Composer) FocusEditor(editor string) bool {
 	c.Lock()
 	defer c.Unlock()
-	if c.editHeaders && c.editor != nil {
-		return
+	if c.editHeaders || c.editor == nil {
+		return false
 	}
-	c.focusEditor(editor)
+	return c.focusEditor(editor)
 }
 
-func (c *Composer) focusEditor(editor string) {
+func (c *Composer) focusEditor(editor string) bool {
 	editor = strings.ToLower(editor)
 	c.focusActiveWidget(false)
+	defer c.focusActiveWidget(true)
 	for i, f := range c.focusable {
 		e := f.(*headerEditor)
 		if strings.ToLower(e.name) == editor {
 			c.focused = i
-			break
+			return true
 		}
 	}
-	c.focusActiveWidget(true)
+	return false
 }
 
 // AddEditor appends a new header editor to the compose window.
@@ -1638,7 +1666,7 @@ func newReviewMessage(composer *Composer, err error) *reviewMessage {
 
 	reviewCmds := []reviewCmd{
 		{":send<enter>", "Send", ""},
-		{":edit<enter>", "Edit", ""},
+		{":edit<enter>", "Edit (body and headers)", ""},
 		{":attach<space>", "Add attachment", ""},
 		{":detach<space>", "Remove attachment", ""},
 		{":postpone<enter>", "Postpone", ""},
@@ -1939,7 +1967,8 @@ func (c *Composer) setTitle() {
 	data.SetHeaders(&header, c.parent)
 
 	var buf bytes.Buffer
-	err := templates.Render(c.acct.UiConfig().TabTitleComposer, &buf,
+	uiConf := c.acct.UiConfig()
+	err := templates.Render(uiConf.TabTitleComposer, &buf,
 		data.Data())
 	if err != nil {
 		c.acct.PushError(err)
