@@ -10,15 +10,11 @@ import (
 	"git.sr.ht/~rockorager/go-jmap/mail/mailbox"
 )
 
-func (w *JMAPWorker) updateFlags(uids []uint32, flags models.Flags, enable bool) error {
+func (w *JMAPWorker) updateFlags(uids []models.UID, flags models.Flags, enable bool) error {
 	var req jmap.Request
 	patches := make(map[jmap.ID]jmap.Patch)
 
 	for _, uid := range uids {
-		id, ok := w.uidStore.GetKey(uid)
-		if !ok {
-			return fmt.Errorf("bug: unknown uid %d", uid)
-		}
 		patch := jmap.Patch{}
 		for kw := range flagsToKeywords(flags) {
 			path := fmt.Sprintf("keywords/%s", kw)
@@ -28,7 +24,7 @@ func (w *JMAPWorker) updateFlags(uids []uint32, flags models.Flags, enable bool)
 				patch[path] = nil
 			}
 		}
-		patches[jmap.ID(id)] = patch
+		patches[jmap.ID(uid)] = patch
 	}
 
 	req.Invoke(&email.Set{
@@ -41,10 +37,44 @@ func (w *JMAPWorker) updateFlags(uids []uint32, flags models.Flags, enable bool)
 		return err
 	}
 
-	return checkNotUpdated(resp)
+	err = checkNotUpdated(resp)
+	if err != nil {
+		return err
+	}
+
+	// If we didn't get an update error, all methods succeeded. We can
+	// update the cache and UI now. We don't update the email state so that
+	// we still grab an updated set from the update channel
+	for _, uid := range uids {
+		jid := jmap.ID(uid)
+		m, err := w.cache.GetEmail(jid)
+		if err != nil {
+			// We'll get this from the update channel
+			continue
+		}
+		if enable {
+			for kw := range flagsToKeywords(flags) {
+				m.Keywords[kw] = true
+			}
+		} else {
+			for kw := range flagsToKeywords(flags) {
+				delete(m.Keywords, kw)
+			}
+		}
+		err = w.cache.PutEmail(jid, m)
+		if err != nil {
+			w.w.Warnf("PutEmail: %s", err)
+		}
+		// Get the UI updated immediately
+		w.w.PostMessage(&types.MessageInfo{
+			Info: w.translateMsgInfo(m),
+		}, nil)
+	}
+
+	return nil
 }
 
-func (w *JMAPWorker) moveCopy(uids []uint32, destDir string, deleteSrc bool) error {
+func (w *JMAPWorker) moveCopy(uids []models.UID, destDir string, deleteSrc bool) error {
 	var req jmap.Request
 	var destMbox jmap.ID
 	var destroy []jmap.ID
@@ -62,13 +92,9 @@ func (w *JMAPWorker) moveCopy(uids []uint32, destDir string, deleteSrc bool) err
 
 	for _, uid := range uids {
 		dest := destMbox
-		id, ok := w.uidStore.GetKey(uid)
-		if !ok {
-			return fmt.Errorf("bug: unknown uid %d", uid)
-		}
-		mail, err := w.cache.GetEmail(jmap.ID(id))
+		mail, err := w.cache.GetEmail(jmap.ID(uid))
 		if err != nil {
-			return fmt.Errorf("bug: unknown message id %s: %w", id, err)
+			return fmt.Errorf("bug: unknown message id %s: %w", uid, err)
 		}
 
 		patch := w.moveCopyPatch(mail, dest, deleteSrc)
@@ -76,7 +102,7 @@ func (w *JMAPWorker) moveCopy(uids []uint32, destDir string, deleteSrc bool) err
 			destroy = append(destroy, mail.ID)
 			w.w.Debugf("destroying <%s>", mail.MessageID[0])
 		} else {
-			patches[jmap.ID(id)] = patch
+			patches[jmap.ID(uid)] = patch
 		}
 	}
 
@@ -161,11 +187,7 @@ func (w *JMAPWorker) handleModifyLabels(msg *types.ModifyLabels) error {
 	patches := make(map[jmap.ID]jmap.Patch)
 
 	for _, uid := range msg.Uids {
-		id, ok := w.uidStore.GetKey(uid)
-		if !ok {
-			return fmt.Errorf("bug: unknown uid %d", uid)
-		}
-		patches[jmap.ID(id)] = patch
+		patches[jmap.ID(uid)] = patch
 	}
 
 	req.Invoke(&email.Set{
